clear;
clc;
close all;

I = im2double(imread('Pics/3.jpg'));

% apply artificial blur 
len = 15%29.5;
theta = 45%87.2;
nsr_est = 0.0001;
psf = fspecial('motion', len, theta);
cfft = psf2otf(psf, size(I));
Ifft = fft2(I);
Ib = real(ifft2(Ifft.*cfft));

% brighten picture
Ibright = Ib .* 5;
Ibright(Ibright > 1) = 1;
[rows, cols, ch] = size(Ibright);

% We are comparing bebluring algorithms for an image of KNOWN motion blur

%% deblurr - wiener deconvolution 
Ideblur_wiener = deconvwnr(Ibright, psf, nsr_est);

%% deblurr - blind deconvolution
test_theta = 30;
test_len = 45;
nsr_est = 0.0001;
INITPSF = fspecial('motion', test_len, test_theta);
[Ideblur_blind P]= deconvblind(Ibright,INITPSF,20,10*sqrt(nsr_est));

%% deblur - ADMM
c = psf;
lambda = 0.0005;

% convolution kernel for Dx and Dy
dy = [-1; 1];
dx = [-1, 1];

Ideblur_admm = zeros(size(Ibright));
for ch = 1:3
    
    p2o = @(x) psf2otf(x, size(Ibright(:,:,ch)));
    o2p = @(x) otf2psf(x);

    convSize = size(Ibright(:,:,ch));

    % precompute OTFs 
    cFT     = p2o(c);
    cTFT    = conj(p2o(c));
    dxFT    = p2o(dx);
    dxTFT   = conj(p2o(dx));
    dyFT    = p2o(dy);
    dyTFT   = conj(p2o(dy));

    % define function handle for 2D convolution
    Afun    = @(x) ifft2( fft2(x).*cFT );
    Atfun   = @(x) ifft2( fft2(x).*cTFT );
    AtAfun  = @(x) Atfun(Afun(x));

    % simulated measurements
    b   = Ibright(:,:,ch);
    bFT = fft2(b);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % ADMM with (anisotropic) TV        
    numItersADMM    = 100;  % number of ADMM iterations
    rho             = 1; 

    x           = zeros(size(Ibright(:,:,ch)));                       % initial guess 
    u           = cat(3,zeros(size(Ibright(:,:,ch))), zeros(size(Ibright(:,:,ch))));     % slack variable
    z           = cat(3,zeros(size(Ibright(:,:,ch))), zeros(size(Ibright(:,:,ch))));     % splitting variable

    grad_fun    = @(x) cat(3,real(ifft2(fft2(x).*dxFT)), real(ifft2(fft2(x).*dyFT)) ); 
    denominator = cTFT.*cFT + rho .* (dxTFT.*dxFT + dyTFT.*dyFT);

    for k=1:numItersADMM

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % x update - inverse filtering: Fourier multiplications and divisions

        v   = z-u;
        v1FT = fft2(v);

        numerator = cTFT.*bFT + rho .* (dxTFT.*v1FT(:,:,1) + dyTFT.*v1FT(:,:,2));       
        x = real(ifft2( numerator ./ denominator));

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % z update - soft shrinkage

        kappa = lambda/rho;
        v = (grad_fun(x)+u); 
        z = max(1 - kappa/abs(v),0) .* v;

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % u update    
        u = u + grad_fun(x)-z;

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % residual

        objective = 0.5*norm(reshape(b-Afun(x),[],1),2)^2 + lambda* sum( reshape(abs(grad_fun(x)),[],1) );
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plot

    Ideblur_admm(:,:,ch) = x;
end

%% Display
figure
subplot(2,2,1)
imshow(Ibright)
title('Image with artif motion blur - Brightened')
subplot(2,2,2)
imshow(Ideblur_wiener)
title('Deblurred - Wiener Deconvolution')
subplot(2,2,3)
imshow(Ideblur_blind)
title('Deblurred - Blind Deconvolution')
subplot(2,2,4)
imshow(Ideblur_admm)
title('Deblurred - ADMM')

%% PSNR for Blind Deconvolution

MSE = 0;
max = 0;
for i = 1:375
    for j = 1:500
        for k = 1:3
            originalVal = Ideblur_wiener(i,j,k); 
            MSE = MSE + (originalVal-Ideblur_blind(i,j,k))^2;
            if originalVal > max
                max = originalVal;
            end
        end 
    end
end
MSE = MSE / (375*500*3);
PSNR = 10*log10(max^2/MSE);