clear;
clc;
close all;

I = im2double(imread('Pics/3.jpg'));

% apply artificial blur 
len = 15;
theta = 45;
nsr_est = 0.0001;
psf = fspecial('motion', len, theta);
cfft = psf2otf(psf, size(I));
Ifft = fft2(I);
Ib = real(ifft2(Ifft.*cfft));

% brighten picture
Ibright = Ib .* 5;
Ibright(Ibright > 1) = 1;
[rows, cols, ch] = size(Ibright);
% imshow(Ibright);

Ideblur_ground_truth = deconvwnr(Ibright, psf, nsr_est);


%% Our algorithm - implementing blind deconvolution using theta/len testing with wiener deconvolution 
% Here we assume the motion blur of the image is NOT KNOWN

% Initialize parameters 
test_len = 15;
nsr_est = 0.0001;
Ideblur = Ibright; 
rough_best_theta = 0;
perf_val = 0;

% look for rough best theta for deconv
for test_theta = 0:10:360
    
    % apply deblur 
    psf_test = fspecial('motion', test_len, test_theta);
    Ideblur_test = deconvwnr(Ibright, psf_test, nsr_est);
    
    % check quality of image (sharpness)
    Ideblur_test_fft = fft2(rgb2gray(Ideblur_test));
    threshold = max(max(abs(Ideblur_test_fft)))/1000;
    Itest_pass = abs(Ideblur_test_fft) < threshold;
    num_thresh_pass = sum(Itest_pass, 'all');
    
        
    % check if current test 
    if num_thresh_pass > perf_val
        perf_val = num_thresh_pass;
        Ideblur = Ideblur_test;
        rough_best_theta = test_theta;
    end
end

% fine tune best theta search
best_theta = rough_best_theta;

for test_theta = rough_best_theta-9:rough_best_theta+9
    
    % apply deblur 
    psf_test = fspecial('motion', test_len, test_theta);
    Ideblur_test = deconvwnr(Ibright, psf_test, nsr_est);
    
    % check quality of image (sharpness)
    Ideblur_test_fft = fft2(rgb2gray(Ideblur_test));
    threshold = max(max(abs(Ideblur_test_fft)))/1000;
    Itest_pass = abs(Ideblur_test_fft) < threshold;
    num_thresh_pass = sum(Itest_pass, 'all');
    
        
    % check if current test 
    if num_thresh_pass > perf_val
        perf_val = num_thresh_pass;
        Ideblur = Ideblur_test;
        best_theta = test_theta;
    end
end

% look for rough best len for deconv
rough_best_len = test_len;
perf_val = 0;
for test_len = 5:5:30
    
    % apply deblur 
    psf_test = fspecial('motion', test_len, best_theta);
    Ideblur_test = deconvwnr(Ibright, psf_test, nsr_est);
    
    % check quality of image (sharpness)
    Ideblur_test_fft = fft2(rgb2gray(Ideblur_test));
    threshold = max(max(abs(Ideblur_test_fft)))/900;
    Itest_pass = abs(Ideblur_test_fft) < threshold;
    Itest_pass = Itest_pass.*(abs(Ideblur_test_fft) > (threshold/200));
    num_thresh_pass = sum(Itest_pass, 'all');
    
        
    % check if current test 
    if num_thresh_pass > perf_val
        perf_val = num_thresh_pass;
        Ideblur = Ideblur_test;
        rough_best_len = test_len;
    end
    
end

% fine tune best len search
best_len = rough_best_len;

for test_len = rough_best_len-4:rough_best_len+4
    
    if test_len == 1
        test_len = 2;
    end
    
    % apply deblur 
    psf_test = fspecial('motion', test_len, best_theta);
    Ideblur_test = deconvwnr(Ibright, psf_test, nsr_est);
    
    % check quality of image (sharpness)
    Ideblur_test_fft = fft2(rgb2gray(Ideblur_test));
    threshold = max(max(abs(Ideblur_test_fft)))/900;
    Itest_pass = abs(Ideblur_test_fft) < threshold;
    Itest_pass = Itest_pass.*(abs(Ideblur_test_fft) > (threshold/200));
    num_thresh_pass = sum(Itest_pass, 'all');
    
        
    % check if current test 
    if num_thresh_pass > perf_val
        perf_val = num_thresh_pass;
        Ideblur = Ideblur_test;
        best_len = test_len;
    end
end


figure
subplot(1,3,1)
imshow(Ibright)
title('Image with artif motion blur - Brightened')
subplot(1,3,2)
imshow(Ideblur_ground_truth)
title('Deblurred - Wiener Deconv (known motion blur)')
subplot(1,3,3)
imshow(Ideblur)
title('Deblurred - Our Algorithm (unknown motion blur)')


%% Denoising

sigma = 1;
filterSize = 2*sigma+1;
averageFilterRadius     = floor(filterSize/2);

% denoise baseline
gauss = fspecial('gaussian',2*sigma+1,sigma);
Ideblur_denoise_base = imfilter(Ideblur, gauss);

% denoise bilateral 
sigmaIntensity = 1.0;
Ideblur_denoise_bilateral = zeros(size(Ideblur));
for c=1:size(Ideblur,3)
    Ideblur_denoise_bilateral(:,:,c) = bilateral(Ideblur(:,:,c), averageFilterRadius, sigma, sigmaIntensity);
end

figure;
subplot(1,3,1);
imshow(Ideblur);
title('Deblurred - Our debluring algorithm');
subplot(1,3,2);
imshow(Ideblur_denoise_base);
title('Deblurred Denoised - Baseline');
subplot(1,3,3);
imshow(Ideblur_denoise_bilateral);
title('Deblurred Denoised - Bilateral');
%% PSNR for Our Algorithm

MSE = 0;
max = 0;
for i = 1:375
    for j = 1:500
        for k = 1:3
            originalVal = Ideblur_ground_truth(i,j,k); 
            MSE = MSE + (originalVal-Ideblur(i,j,k))^2;
            if originalVal > max
                max = originalVal;
            end
        end 
    end
end
MSE = MSE / (375*500*3);
PSNR = 10*log10(max^2/MSE);
