function Ifilt = nlm(I, searchWindowRadius, averageFilterRadius, sigma, nlm_sigma)
    
    % pad image to reduce boundary artifacts
    I = padarray(I,[averageFilterRadius averageFilterRadius],'symmetric');

    % initialize filtered image as 0
    Ifilt = zeros(size(I));
    
    % smoothing kernel
    kernel = fspecial('gaussian', [2*averageFilterRadius+1 2*averageFilterRadius+1], sigma);
        
    for ky= 1+averageFilterRadius:size(I,1)-averageFilterRadius
        for kx= 1+averageFilterRadius:size(I,2)-averageFilterRadius
            
         
            % extract current window
            currentWindow = I(  ky-averageFilterRadius:ky+averageFilterRadius, ...
                                kx-averageFilterRadius:kx+averageFilterRadius);
                            
            % accumulated normalization factor
            normalizationFactor = zeros([1 1 size(I,3)]);
            
            % save maximum weight
            weight_max = 0;

            for yy=ky-searchWindowRadius:ky+searchWindowRadius
                for xx=kx-searchWindowRadius:kx+searchWindowRadius
                    
                    ky_start    = yy-averageFilterRadius;
                    ky_end      = yy+averageFilterRadius;
                    kx_start    = xx-averageFilterRadius;
                    kx_end      = xx+averageFilterRadius;

                    if  (ky_start>0) && (ky_end<=size(I,1)) ...
                            && (kx_start>0) && (kx_end<=size(I,2)) ...
                            && (yy~=ky) && (xx~=kx)

                        % extract other window
                        otherWindow = I(    ky_start:ky_end, ...
                                            kx_start:kx_end);
                        
                        % l2 distance between the two neighborhoods
                        d = sum(sum( kernel .* (currentWindow-otherWindow) .* (currentWindow-otherWindow) ));
                        %d = sum(sum( (currentWindow-otherWindow).^2 ));
                        
                        % compute weight
                        weight = exp( -d ./ (nlm_sigma^2) );
                        
                        if weight > weight_max
                            weight_max = weight;
                        end
                        
                        % add to normalization factor
                        normalizationFactor = normalizationFactor + weight;

                        % add to filtered image
                        Ifilt(ky,kx) = Ifilt(ky,kx) + weight .* I(yy,xx);                                
                    end

                end
            end
            
            % this one makes it a bit better
            % add current pixel as well with max weight            
            normalizationFactor = normalizationFactor + weight_max;
            Ifilt(ky,kx) = Ifilt(ky,kx) + weight_max .* I(ky,kx);   

            % normalize pixel value
            Ifilt(ky,kx) = Ifilt(ky,kx) ./ normalizationFactor;

        end
    end
    
    % crop black boundary
    Ifilt = Ifilt(averageFilterRadius+1:end-averageFilterRadius, averageFilterRadius+1:end-averageFilterRadius);
end
